--------------------------------------------------------------------------------
-- Copyright  2021								|
-- All Rights Reserved.								|
-- 										|
-- Permission to use, copy, modify, and distribute this software in any way	|
-- is not granted without expressed written agreement. This copyright applies	|
-- to the all code, data and artwork related to this project and all associated	|
-- projects published by the author.						|
--										|
-- More information on copyright is available from sources on-line.		|
-- Use an actual factual one!							|
--------------------------------------------------------------------------------
--[[
-- SDPhantom
hooksecurefunc(HelpTipTemplateMixin, "Init", function(self, parent, info, relregion)
	if info.system=="MicroButtons" then
		info.targetPoint=HelpTip.Point.BottomEdgeCenter
		self:AnchorAndRotate()
	end
end)

-- Xrystal (origional version)
if HelpTip and HelpTip.framePool then
	hooksecurefunc(HelpTip,"Show", function(parent,info,relativeRegion)
		for frame in HelpTip.framePool:EnumerateActive() do
			if frame.info.system == "MicroButtons" then
--				frame.info.targetPoint = HelpTip.Point.BottomEdgeCenter
				frame.info.targetPoint = HelpTip.Point.RightEdgeCenter
			end
		end
	end)
end
]]--
	------------------------------------------------------------------------------------------------------------
	--					Main background framework					  --
	------------------------------------------------------------------------------------------------------------

local AddonName, NS = ...
local L = NS.TEXTS

local ceil, floor = math.ceil, floor
local clip, upper = string.trim, string.upper
local sub = string.sub

local BaseHeight = floor(NS.UIH / 5)
local CaveHeight = floor(NS.UIH / 8.2)
local CaveWidth = floor(NS.UIW / 9.75)
local TowerWidth = floor(NS.UIW / 4.5)
local Metrics = {
	BorderWidth = 5.4,
	
	UIW = NS.UIW, 
	UIH = NS.UIH,

	AlertY = floor(NS.UIH*0.3),
	BaseHeight = BaseHeight, -- 20% Overall height for bottom framework
	CaveHeight = CaveHeight,
	CaveHotSpotX = floor(CaveWidth * 0.16),
	CaveHotSpotY = floor(CaveHeight * 0.15),
	CaveWidth = CaveWidth,
	HotSpotSize = floor(NS.UIH * 0.0130), -- ??? Test
	InnerHotSpotX = floor(TowerWidth * 0.17),
	IRefY = floor(NS.UIH*0.03),
	LowerHotSpotY = floor(BaseHeight * 0.335),

	MaxUserHotSpots = 8,

	MinimapHeight = floor(NS.UIH*0.106),
	MinimapWidth = floor(NS.UIH*0.106) + 3,

	MinimapBorderHeight = floor(NS.UIH*0.106) + 3,
	MinimapBorderWidth = floor(NS.UIH*0.106) + 20,

	RampartTop = ceil(BaseHeight * 0.625),

	TowerWidth = TowerWidth, -- Tower widths
	TrayHeight = floor(BaseHeight / 9.7),
	TrinketSize = floor(BaseHeight / 8.5),
}

--[[
function Fizz(setting, value, div)
	local a = Metrics[setting]
	if div then
		print(a / value)
	else
		print(a * value)
	end
end
]]--
NS:AddOption("Metrics", Metrics)

local InlineTextWidth

local function HotSpotOnEnter(self)
	if self.OnReset and not self.Reset then
		self.Reset = true
		self:OnReset()
	end
	self.Text:Show()
end

local function HotSpotOnLeave(self)
	self.Text:Hide()
end

local FrameWork = {
	Base = {
		{
			[1] = {
				point = "BOTTOMLEFT", 
				relative = UIParent, 
			},
			[2] = {
				point = "TOPRIGHT", 
				relative = UIParent, 
				topoint = "BOTTOMLEFT", 
				x = Metrics.TowerWidth, 
				y = Metrics.BaseHeight,
			},
		},
		{
			[1] = {
				point = "TOPLEFT", 
				stack = "Base",
				relative = 1, 
				topoint = "TOPRIGHT",
			},
			[2] = {
				point = "BOTTOMRIGHT", 
				relative = UIParent, 
				topoint = "BOTTOM", 
			},
			parent = {
				stack = "Base",
				frame = 1,
			},
		},
		{
			name = 4,
			[1] = {
				point = "BOTTOMRIGHT", 
				relative = UIParent, 
			},
			[2] = {
				point = "TOPLEFT", 
				relative = UIParent, 
				topoint = "BOTTOMRIGHT", 
				x = -Metrics.TowerWidth,
				y = Metrics.BaseHeight, 
			},
			parent = {
				stack = "Base",
				frame = 1,
			},
		},
		{
			name = 3,
			[1] = {
				point = "TOPRIGHT", 
				stack = "Base",
				relative = 4, 
				topoint = "TOPLEFT",
			},
			[2] = {
				point = "BOTTOMLEFT", 
				relative = UIParent, 
				topoint = "BOTTOM",
			},
			parent = {
				stack = "Base",
				frame = 1,
			},
		},
		{
			name = "Cave",
			[1] = {
				point = "BOTTOM", 
				relative = UIParent, 
--				y = Metrics.BaseHeight/1.685,
				y = Metrics.BaseHeight*0.582,
			},
			parent = {
				stack = "Base",
				frame = 1,
			},
			size={ w = Metrics.CaveWidth, h = Metrics.CaveHeight },
			coords = {0, 1, 0.31, 1},
		},
		{
			name="MinimapGlass",
			[1] = {
--				point="CENTER", 
--				relative = Minimap, 
				point="TOPLEFT", 
				relative = Minimap, 
			},
			[2] = {
				point="BOTTOMRIGHT", 
				relative = Minimap, 
			},
			parent = {
				stack = "Base",
				frame = 1,
			},
			strata = "MEDIUM",
			level = 4,
		},
		{
			name = "Minimap",
			[1] = {
				point = "BOTTOM", 
				relative = UIParent, 
				y = Metrics.BorderWidth,
			},
			parent = {
				stack = "Base",
				frame = 1,
			},
			strata = "MEDIUM",
			level = 5,
			size={ w = Metrics.MinimapBorderWidth, h = Metrics.MinimapBorderHeight+3 },
			coords = { 0.35499999022484, 0.95, 0.1039000018239, 0.63800001144409 },
		},
	},
	HotSpots = {
		[1] = { -- Zone ability
			[1] = {
				point = "BOTTOMLEFT",
				stack = "Base",
				relative = 1,
			},
			text = {
				point = "BOTTOMLEFT",
				topoint = "BOTTOMRIGHT",
				x = 5,
			},
		},
		[2] = { -- XCalc local
			[1] = {
				point = "TOPLEFT",
				stack = "Base",
				relative = 1,
			},
			text = {
				point = "TOPLEFT",
				topoint = "TOPRIGHT",
				x = 5,
			},
		},
		[3] = { -- TinyPad
			[1] = {
				point = "TOPRIGHT",
				stack = "Base",
				relative = 1,
				x = -Metrics.InnerHotSpotX,
			},
			text = {
				point = "TOPRIGHT",
				topoint = "TOPLEFT",
				x = -5,
			},
		},
		[4] = { -- ToT Toggle
			[1] = {
				point = "TOPRIGHT",
				stack = "Base",
				relative = 1,
				x = -Metrics.InnerHotSpotX,
				y = -Metrics.LowerHotSpotY,
			},
			text = {
				point = "BOTTOMLEFT",
				topoint = "BOTTOMRIGHT",
				x = 5,
			},
		},
		[5] = { -- Focus Toggle
			[1] = {
				point = "TOPLEFT",
				stack = "Base",
				relative = 4,
				x = Metrics.InnerHotSpotX,
				y = -Metrics.LowerHotSpotY,
			},
			text = {
				point = "BOTTOMRIGHT",
				topoint = "BOTTOMLEFT",
				x = -5,
			},
		},
		[6] = { -- Map
			[1] = {
				point = "TOPLEFT",
				stack = "Base",
				relative = 4,
				x = Metrics.InnerHotSpotX,
			},
			text = {
				point = "TOPLEFT",
				topoint = "TOPRIGHT",
				x = 5,
			},
		},
		[7] = { -- Bags
			[1] = {
				point = "TOPRIGHT",
				stack = "Base",
				relative = 4,
			},
			text = {
				point = "TOPRIGHT",
				topoint = "TOPLEFT",
				x = -5,
			},
		},
		[8] = { -- Meters
			[1] = {
				point = "BOTTOMRIGHT",
				stack = "Base",
				relative = 4,
			},
			text = {
				point = "BOTTOMRIGHT",
				topoint = "BOTTOMLEFT",
				x = -5,
			},
		},
	-- Cave/Target HotSpots
		[9] = { -- Inspect
			[1] = {
				point = "BOTTOMLEFT",
				stack = "Base",
				relative = "Cave",
				x = Metrics.CaveHotSpotX,
				y = Metrics.CaveHotSpotY,
			},
			text = {
				point = "BOTTOMRIGHT",
				topoint = "BOTTOMleft",
				x = -5,
				L = L.HotSpot9,
			},
			func = function()
					if Examiner then
						if Examiner.model:IsVisible() then
							HideUIPanel(Examiner)
						else
							InspectUnit("target")
						end
					else
						if not InspectFrame or not InspectFrame:IsShown() then
							InspectUnit("target")
						else
							HideUIPanel(InspectFrame)
						end
					end  
				end,
		},
		[10] = { -- Trade
			[1] = {
				point = "TOPLEFT",
				stack = "Base",
				relative = "Cave",
				x = Metrics.CaveHotSpotX,
			},
			text = {
				point = "TOPLEFT",
				topoint = "TOPRIGHT",
				x = 5,
				L = L.HotSpot10,
			},
			func=function()
					InitiateTrade("target")
				end,
			
		},
		[11] = { -- Whisper
			[1] = {
				point = "TOPRIGHT",
				stack = "Base",
				relative = "Cave",
				x = -Metrics.CaveHotSpotX,
			},
			text={
				point = "TOPRIGHT",
				topoint = "TOPLEFT",
				x = -5,
				L = L.HotSpot11,
			},
			func = function()
					local TargetName, TargetRealm = UnitName("target") 
					if TargetRealm then TargetName = TargetName.."-"..TargetRealm end 
					ChatFrame_SendTell(TargetName)
				end,
		},

		[12] = { -- Follow
			[1] = {
				point = "BOTTOMRIGHT",
				stack = "Base",
				relative = "Cave",
				x = -Metrics.CaveHotSpotX,
				y = Metrics.CaveHotSpotY,
			},
			text={
				point = "BOTTOMLEFT",
				topoint = "BOTTOMRIGHT",
				x = 5,
				L = L.HotSpot12,
			},
			func = function()
					FollowUnit("target")
				end,
		},
	},
	Texts = {
		[1] = { -- Armor/Durability
			[1] = {
				point = "BOTTOMLEFT",
				stack = "HotSpots",
				relative = 2,
				topoint = "TOPLEFT",
				x = 5,
			},
			parent = {
				stack = "Base",
				frame = 1,
			},
			text = {
				fontsize = NS.FontSize + 1,
			},
		},
		[2] = { -- Position
			[1] = {
				point = "BOTTOM",
				relative = UIParent,
				y = -1,
			},
			parent={
				stack = "Base",
				frame = "Minimap",
			},
			text = {
				fontsize = NS.FontSize - 3,
			},
		},
		[3] = { -- Zone
			[1] = {
				point = "TOPLEFT",
				stack = "HotSpots",
				relative = 6,
				topoint = "BOTTOMRIGHT",
			},
			[2] = {
				point = "TOPRIGHT",
				stack = "HotSpots",
				relative = 7,
				topoint = "BOTTOM",
			},
			layer = "OVERLAY",
			sublayer = 5,
			parent = {
				stack = "Base",
				frame = 4,
			},
			text = {
				justifyh = "CENTER",
			},
		},
		[4] = { -- Latency
			[1] = {
				point = "TOPLEFT",
				stack = "Texts",
				relative = 3,
				topoint = "BOTTOMLEFT",
				inlinetext = true,
				y = -(Metrics.TrayHeight+1),
			},
			parent = {
				stack = "Base",
				frame = 4,
			},
			text = {
				justifyh = "CENTER",
			},
		},
		[5] = { -- FPS
			[1] = {
				point = "LEFT",
				stack = "Texts",
				relative = 4,
				topoint = "RIGHT",
				inlinetext = true,
			},
			parent = {
				stack = "Base",
				frame = 4,
			},
			text = {
				justifyh = "CENTER",
			},
		},
		[6] = { -- Time
			[1] = {
				point = "LEFT",
				stack = "Texts",
				relative = 5,
				topoint = "RIGHT",
				inlinetext = true,
			},
			parent = {
				stack = "Base",
				frame = 4,
			},
			text = {
				justifyh = "CENTER",
			},
		},
		[7] = { -- Gold
			[1] = {
				point = "LEFT",
				stack = "Texts",
				relative = 6,
				topoint = "RIGHT",
				inlinetext = true,
				multi = 2,
			},
			parent={
				stack = "Base",
				frame = 4,
			},
			text={
				justifyh = "CENTER",
			},
		},
	},
	Utility = {
		[1] = {
			name = "ButtonTray1", -- Possibly two or more wheel scrollable
			[1] = {
				point = "TOPLEFT", 
				stack = "Texts",
				relative = 3,
				topoint = "BOTTOMLEFT",
			},
			[2] = {
				point = "BOTTOMRIGHT", 
				stack = "Texts",
				relative = 3,
				topoint = "BOTTOMRIGHT",
--				x = Metrics.HotSpotSize,
				y = -Metrics.TrayHeight,
			},
			parent = {
				stack = "Base",
				frame = 4,
			},
		},
		[2] = {
			name = "MiniButton", 
			size = { w = Metrics.TrayHeight+2, h = Metrics.TrayHeight+2 },
			[1] = {
				point = "CENTER", 
				stack = "Utility",
				relative = "ButtonTray1",
			},
			widget = "Button",
			parent = {
				stack = "Utility",
				frame = "ButtonTray1",
			},
		},
		[3] = {
			name = "Logs",
			[1] = {
				point = "TOPLEFT", 
				stack = "HotSpots",
				relative = 5,
				topoint = "BOTTOM",
				x = 4,
				y = -4,
			},
			[2] = {
				point = "BOTTOMRIGHT", 
				stack = "HotSpots",
				relative = 8,
				topoint = "CENTER",
			},
			parent = {
				stack = "Base",
				frame = 4,
			},
		},
		[4] = {
			name = "Meters",
			[1] = {
				point = "TOPLEFT", 
				stack = "HotSpots",
				relative = 5,
				topoint = "BOTTOM",
			},
			[2] = {
				point = "BOTTOMRIGHT", 
				stack = "HotSpots",
				relative = 8,
				topoint = "CENTER",
			},
			parent = {
				stack = "Base",
				frame = 4,
			},
			hide=true,
		},
		[5] = {
			name = "Downloading", -- May not be needed if StreamingIcon does this also but better
			[1] = {
				point = "BOTTOMLEFT", 
				stack = "Texts",
				relative = 4, -- Latency
				topoint = "TOPLEFT",
				y = -2,
			},
			[2] = {
				point = "BOTTOMRIGHT", 
				stack = "Texts",
				relative = 4, -- Latency
				topoint = "TOPRIGHT",
				y = -2,
			},
			parent = {
				stack = "Base",
				frame = 4,
			},
		},
		[6] = {
			name = "Trinket1",
			[1] = {
				point = "BOTTOMLEFT",
				stack = "HotSpots",
				relative = 5,
				topoint = "BOTTOMRIGHT",
				x = 0,
				y = 0,
			},
			id = 13,
			size = { w = Metrics.TrinketSize, h = Metrics.TrinketSize, },
			widget = "Button",
			templates = "SecureActionButtonTemplate",
		},
		[7] = {
			name = "Trinket2",
			[1] = {
				point = "LEFT",
				stack = "Utility",
				relative = "Trinket1",
				topoint = "RIGHT",
				x = 2, 
				y = 0,
			},
			id = 14,
			size = { w = Metrics.TrinketSize, h = Metrics.TrinketSize, },
			widget = "Button",
			templates = "SecureActionButtonTemplate",
		},
	},
	AssetsCommon = {
		[1] = { -- QuestTrackerDragFrame, see AssetsCommon in FramesInitialise.lua
		},
		[2] = {
			name = "Minimap",
			frame = Minimap,
			[1] = {
				point = "BOTTOM", 
				relative = UIParent,
				topoint = "BOTTOM",
				y = Metrics.BorderWidth,
			},
			size = { w = Metrics.MinimapWidth, h = Metrics.MinimapHeight },
			strata = "BACKGROUND",
			level = 2,
		},
		[3] = {
			name = "AlertFrame",
			frame = AlertFrame,
			[1] = {
				point = "BOTTOM", 
				relative = UIParent,
				y = Metrics.AlertY,
			},
		},
		[4] = {
			name = "ItemRefTooltip",
			frame = ItemRefTooltip,
			[1] = {
				point = "TOP", 
				relative = UIParent,
				y = -Metrics.IRefY,
			},
		},
		[5] = {
			name = "ItemRefTooltip",
		},
		[6] = {
			name = "QuestTimerFrame",
			frame = QuestTimerFrame,
			versionspecific = true,
			[1] = {
				point = "TOP", 
				relative = UIParent,
				y = -150,
			},
		},
		[7] = {
			name = "StreamingIcon",
			frame = StreamingIcon,
			[1] = {
				point = "BOTTOM", 
				stack = "Texts",
				relative = 4,
				topoint = "BOTTOMLEFT",
				y = -10,
-- downloading
--/run StreamingIcon_OnEvent(StreamingIcon, "STREAMING_ICON", 0) -- Fade out and stop
--/run StreamingIcon_OnEvent(StreamingIcon, "STREAMING_ICON", 1) -- Green
--/run StreamingIcon_OnEvent(StreamingIcon, "STREAMING_ICON", 2) -- Yellow
--/run StreamingIcon_OnEvent(StreamingIcon, "STREAMING_ICON", 3) -- Red
			},
		},
		[8] = {
			name = "LossOfControlFrame",
			frame = LossOfControlFrame,
			versionspecific = true,
		},
	},
	Meters = {
	},
}

if NS:IsValidVersion(NS.WOW_Project.Cataclysm_Classic) then
	for i=1, #FrameWork.Texts do
		if FrameWork.Texts[i].text.fontsize then
			FrameWork.Texts[i].text.fontsize = FrameWork.Texts[i].text.fontsize - 3
		else
			FrameWork.Texts[i].text.fontsize = NS.FontSize - 3
		end
	end
	FrameWork.Utility[6][1].y = - 3
	for i=6, 7 do
		for k,_ in pairs(FrameWork.Utility[i].size) do
			FrameWork.Utility[i].size[k] = Metrics.TrinketSize - 2
		end
	end
end

if NS:IsValidVersion(NS.WOW_Project.WrathClassic) then
--	for k, v in pairs(FrameWork.Texts) do
	for i=1, #FrameWork.Texts do
		if FrameWork.Texts[i].text.fontsize then
			FrameWork.Texts[i].text.fontsize = FrameWork.Texts[i].text.fontsize - 2
		else
			FrameWork.Texts[i].text.fontsize = NS.FontSize - 2
		end
	end
	FrameWork.Utility[6][1].y = - 2
	for i=6, 7 do
		for k,_ in pairs(FrameWork.Utility[i].size) do
			FrameWork.Utility[i].size[k] = Metrics.TrinketSize - 1
		end
	end
end
if NS:IsValidVersion(NS.WOW_Project.Classic) then
	FrameWork.Texts[2].text.fontsize = NS.FontSize - 1
	for i=6, 7 do
		for k,_ in pairs(FrameWork.Utility[i].size) do
			FrameWork.Utility[i].size[k] = 30
		end
	end
end
--if MicroButtonAndBagsBar then
if GetExpansionLevel() > 7 then
	FrameWork.Texts[4][1].y = FrameWork.Texts[4][1].y - 2
end

local function CreateBaseFrame(tbl)
	local f = CreateFrame(tbl.widget or "Frame", tbl.name and tbl.name or nil, NS:GetFrameParent(tbl), tbl.templates)
	if tbl.hide then
		f:Hide()
	end

	if tbl.strata then
		f:SetFrameStrata(tbl.strata)
	end
	if tbl.level then
		f:SetFrameLevel(tbl.level)
	end
	if tbl.id then
		f:SetID(tbl.id)
	end
	return f
end

local function CreateBase(stack, tbl, id)
	local f = NS:AddFrame(stack, tbl.name or id, CreateBaseFrame(tbl))
	f.Texture = f:CreateTexture()
	f.Texture:SetAllPoints()
	function f:SetTexture(texture) self.Texture:SetTexture(texture) end
	function f:SetColor(r, g, b, a) self.Texture:SetVertexColor(r, g, b, a) end
	if tbl.coords then
		f.Texture:SetTexCoord(unpack(tbl.coords))
	end
	return f
end

local function CreateHotSpot(stack, tbl, id)
	local Parent = NS:GetFrame("Base", tbl[1].relative)
	local f = NS:AddFrame(stack, id, CreateFrame("Button", nil, Parent))
	f.id = id
	f:SetSize(Metrics.HotSpotSize, Metrics.HotSpotSize)
	f.Text = f:CreateFontString()
	f.Text:SetFont(NS.Font, NS.FontSize, "")
	f.Text:Hide()
	f:SetScript("OnEnter", HotSpotOnEnter)
	f:SetScript("OnLeave", HotSpotOnLeave)
	f.SetText = function(self, text) self.Text:SetText(text) end
	if id > Metrics.MaxUserHotSpots then
		f.Text:SetText(L["HotSpot"..id])
	end
	if tbl.func then
		f:SetScript("OnClick", tbl.func)
	end
	return f
end

local function CreateText(stack, tbl, id)
	if id == 4 then
		InlineTextWidth = floor(NS:GetFrame(stack, 3):GetWidth()/5) -- Zone text width/5
	end
	local Parent = NS:GetFrameParent(tbl)
	local f = NS:AddFrame(stack, id, Parent:CreateFontString(nil))
	if tbl.text.fontobject then
		f:SetFontObject(tbl.text.fontobject)
	else
		f:SetFont(tbl.text.font or NS.Font, tbl.text.fontsize or NS.FontSize, tbl.text.flags)
	end
	if tbl.text.color then
		f:SetTextColor(unpack(tbl.text.color))
	else
		f:SetTextColor(1, 0.82, 0)
	end
	if tbl.text.justifyh then
		f:SetJustifyH(tbl.text.justifyh)
	end
	NS:SetPoints(f, tbl)
	if tbl[1].inlinetext then -- table key 1 has inlinetext
		f:SetWidth(InlineTextWidth * (tbl[1].multi or 1)) -- and multi
	end
	return f
end

local function CreateUtility(stack, tbl, id)
	local f = NS:AddFrame(stack, tbl.name or id, CreateBaseFrame(tbl))
	return f
end

local function AssetsCommon(stack, tbl, id)
	local f
	if tbl.versionspecific then
		f = tbl.frame
		if not f then return end
	end
	local f = tbl.frame or CreateBaseFrame(tbl)
	if tbl.record then
		NS:AddFrame(stack, tbl.name or f:GetName(), f)
	end
	return f
end

local function AssetsRetail(stack, tbl, id)
	local f = tbl.frame
	if tbl.record then
		NS:AddFrame(stack, tbl.name or id, f)
	end
	if tbl.justifyh then
		f:SetJustifyH(tbl.justifyh)
	end
	return f
end

local function MeterInit(stack, tbl, id)
end

local Order = {
	{ manifest="Base", Strata="BACKGROUND", Level=1, create=CreateBase },
	{ manifest="HotSpots", Strata="BACKGROUND", Level=4, create=CreateHotSpot },
	{ manifest="Texts", Layer="ARTWORK", SubLayer=1, create=CreateText },
	{ manifest="Utility", Strata="BACKGROUND", Level=3, create=CreateUtility },
	{ manifest="AssetsCommon", Strata="BACKGROUND", Level=2, create=AssetsCommon },
	{ manifest="Meters", create=MeterInit },
}

local function InitialiseFramework(self)
	local f, setup, tbl, Strata, Level, Layer, SubLayer
	for load=1, #Order do
		setup = Order[load]
		Strata, Level, Layer, SubLayer = NS:GetFrameStack(setup)
		for i=1, #FrameWork[setup.manifest] do -- Create Frames
			tbl = FrameWork[setup.manifest][i]
			f = setup.create(setup.manifest, tbl, i)
			if f then
				if tbl.size then
					f:SetSize(tbl.size.w, tbl.size.h)
				end
				NS:SetFrameStack(f, tbl, Strata, Level, Layer, SubLayer)
				NS:SetPoints(f, tbl)
				NS:InitialiseFrame(f, setup.manifest, tbl.name or i)
			end
		end
	end
end

NS:RegisterForInit("ADDON_LOADED", "InitialiseFramework", 0, InitialiseFramework)

local function GetHotSpotSettings(id)
	local Settings = NS:GetSettings("UI").hotspots[id]
	if type(Settings) == "number" then
		Settings = NS:GetSettings(nil, "Defaults").UI.hotspots[Settings]
	end
	return Settings
end
local function SetHotSpotLabel(id, frame) --, settings)
	local Settings = GetHotSpotSettings(id)
	local f = frame or NS:GetFrame("HotSpots", id)
	f:SetText(Settings.label)
end
NS:AddOption("SetHotSpotLabel", SetHotSpotLabel)

local function ToggleMeterFrame(self, action)
	if not self.Logs then
		self.Logs = NS:GetFrame("Utility", "Logs")
		self.Meters = NS:GetFrame("Utility", "Meters")
	end
	local MetersShown = self.Meters:IsShown()
	if action then
		if action == "show" then
			MetersShown = false
		else
			MetersShown = true
		end
	end
	self.Logs.anim:SetFromAlpha(MetersShown and 0 or 1)
	self.Logs.anim:SetToAlpha(MetersShown and 1 or 0)
	self.Meters.anim:SetFromAlpha(MetersShown and 1 or 0)
	self.Meters.anim:SetToAlpha(MetersShown and 0 or 1)
	self.Logs:Show()
	self.Meters:Show()
	self.Logs.ShowAction = MetersShown
	self.Meters.ShowAction = not MetersShown
	self.Meters.ag:Play()
	self.Logs.ag:Play()
end

local function SetHotSpotOnClick(id, frame, settings)
	local Settings = GetHotSpotSettings(id)
	local f = frame or NS:GetFrame("HotSpots", id)
	if clip(Settings.script) ~= "" then
		FauxMazzleFrame["HS" .. id .. "OnClick"] = nil
		FauxMazzleFrame["HS" .. id .. "OnReset"] = nil
		f.ClickFunc = nil
		f.OnReset = nil
		f:SetScript("OnClick", nil)
		if not Settings.addon or Settings.addon and IsAddOnLoaded(Settings.addon) then
			RunScript("function FauxMazzleFrame.HS" .. id .. "OnClick(self)" .. clip(Settings.script) .. " end")
			f.ClickFunc = FauxMazzleFrame["HS" .. id .. "OnClick"]
			if Settings.onreset then
				RunScript("function FauxMazzleFrame.HS" .. id .. "OnReset(self)" .. clip(Settings.onreset) .. " end")
				f.OnReset = FauxMazzleFrame["HS" .. id .. "OnReset"]
			end
			f:SetScript("OnClick", function(self) self.ClickFunc(self) end)
		end
	end
end
NS:AddOption("SetHotSpotOnClick", SetHotSpotOnClick)

local function ConfigureHotSpots(self)
--	local Settings = NS:GetSettings("UI").hotspots
	for i=1, Metrics.MaxUserHotSpots do
		local f = NS:GetFrame("HotSpots", i)
		if i == 8 then
			f.ToggleMeterFrame = ToggleMeterFrame
		end
		SetHotSpotLabel(i, f)--, Settings[i])
		SetHotSpotOnClick(i, f)--, Settings[i])
	end
end

NS:RegisterForInit("PLAYER_LOGIN", "ConfigureHotSpots", 1, ConfigureHotSpots)

local function SetMeterPoints(frame, anchor, meter, settings)-- maybe change settings to layout, offsets
	local points = {
		LEFT = {
			TOPLEFT = {},
			BOTTOMRIGHT = { topoint="BOTTOM", x=-1 },
		},
		RIGHT = {
			TOPLEFT = { topoint="TOP", x=1 },
			BOTTOMRIGHT = {},
		},
		FULL = {
			TOPLEFT = {},
			BOTTOMRIGHT = {},
		},
	}
	local defaults = {
		damage = "RIGHT",
		threat = "LEFT",
	}
	local Layout = defaults[meter]
	if type(settings) == "string" then
		Layout = upper(settings)
	end
	frame:ClearAllPoints()
	Layout = points[Layout]
	for k, v in pairs(Layout) do
		local x = v.x or 0
		local y = v.y or 0
		if settings.Offsets then
			for p, t in pairs(settings.Offsets) do
				if upper(p) == k then
					x = t.x or x
					y = t.y or y
					break
				end
			end
		end
		if not frame.fmSetPoint then
			frame.fmSetPoint = frame.SetPoint
			frame.SetPoint = function() end
		end
		frame:fmSetPoint(k, anchor, v.topoint or k, x, y)	
	end
end

local function PositionMeters(self)
	local Settings = NS:GetSettings("Addons").Meters --NS:GetSettings("Meters")
	if Settings.inactive then
		return
	end
	local MeterCount = 0
	if Settings.damage and not Settings.damage.inactive then
		MeterCount = MeterCount + 1
	end
	if Settings.threat and not Settings.threat.inactive then
		MeterCount = MeterCount + 1
	end
	if MeterCount == 0 then 
		return 
	end
	local MeterAnchor = NS:GetFrame("Utility", "Meters")
	for metertype, v in pairs(Settings) do
		MeterAnchor:KillMeter(metertype)
		local AddonSettings = type(v) == "table" and NS.GetAccessorySettings(v.addon, "Meter", metertype)
		if AddonSettings then
			local f
			if AddonSettings.Frame then
				f = _G[AddonSettings.Frame]
			end
			if f then
				local oldf = NS:GetFrame("Meters", metertype, true) -- Suppressed GetFrame warning!
				if oldf then
					oldf:SetParent(UIParent)
					oldf:ClearAllPoints()
					oldf:SetPoint("BOTTOM", UIParent, "TOP", 50, 0)
					if oldf.OldOnDragStart then
						oldf:SetScript("OnDragStart", oldf.OldOnDragStart)
					end
					oldf:SetResizable(oldf.OldResizable)
				end
				NS:AddFrame("Meters", metertype, f, true)
				f:SetParent(MeterAnchor)
				if MeterCount == 1 then
					SetMeterPoints(f, MeterAnchor, metertype, "Full")
				else
--					SetMeterPoints(f, MeterAnchor, metertype, FauxMazzleUI[v.addon])
					SetMeterPoints(f, MeterAnchor, metertype, AddonSettings)
				end
				MeterAnchor:AddMeter(metertype, f)
				f.OldOnDragStart = f:GetScript("OnDragStart")
				f:SetScript("OnDragStart", nil)
				f.OldResizable = f:IsResizable()
				f:SetResizable(false)
				f:Show()
			end
		end
	end
end
NS:AddOption("PositionMeters", PositionMeters)

NS:RegisterForInit("PLAYER_LOGIN", "PositionMeters", 3, PositionMeters)

local function TalkingHeadPos()
	if not TalkingHeadFrame then
		return
	end
	local Settings = NS:GetSettings("LOD").TalkingHeadFrame
	if not Settings then 
		NS:GetSettings("LOD").TalkingHeadFrame = { point="IGNORE", }
		Settings = NS:GetSettings("LOD").TalkingHeadFrame
	end
	TalkingHeadFrame.ignoreFramePositionManager = true -- important
	if not TalkingHeadFrame.FMHooked then
		TalkingHeadFrame.FMHooked = true
		TalkingHeadFrame:SetScale(0.6)
--		hooksecurefunc(TalkingHeadFrame, "SetPoint", function(self)
		local function th(self)
			local Settings = NS:GetSettings("LOD").TalkingHeadFrame
			if Settings.point == "IGNORE" then
				return
			end
			NS.PointFrame.ClearAllPoints(TalkingHeadFrame)
			if sub(Settings.point, 1, 6) ~= "BOTTOM" then
				NS.PointFrame.SetPoint(TalkingHeadFrame, "TOP", UIParent, Settings.point, Settings.x, Settings.y)
			else
				NS.PointFrame.SetPoint(TalkingHeadFrame, "BOTTOMLEFT", NS:GetFrame("HotSpots", 6), "TOPLEFT", 0, 0)
			end
		end
		hooksecurefunc(TalkingHeadFrame.MainFrame.TalkingHeadsInAnim, "Play", th)
		th(TalkingHeadFrame)
	end
end

NS:AddOption("TalkingHeadPos", TalkingHeadPos)

NS:RegisterForInit("PLAYER_LOGIN", "TalkingHeadPos", 3, TalkingHeadPos)

-----------------------------
-- LOD addon configuration --
-----------------------------

local LODFrames = {
	Blizzard_ArchaeologyUI = {
		OnLoad=function()
			ArcheologyDigsiteProgressBar.ignoreFramePositionManager = true
			ArcheologyDigsiteProgressBar:ClearAllPoints()
			ArcheologyDigsiteProgressBar:SetPoint("TOPRIGHT", NS:GetFrame("Base", "Minimap"), "TOPLEFT", 25, 0)
		end,
	},
}

local LODCount = 0
for k, v in pairs(LODFrames) do
	local name, _, _, loaded, loadable = GetAddOnInfo(k)
	if name and not loaded and loadable == "DEMAND_LOADED" then
		LODCount = LODCount + 1
	end
end

local LodFrame = CreateFrame("Frame")
local function ProcessLOD(self, event, addon)
	if LODFrames[addon] then
		LODFrames[addon].OnLoad()
		LODFrames[addon] = nil
		LODCount = LODCount - 1
	end
	if LODCount == 0 then
		NS:UnregisterEvent("ADDON_LOADED", LodFrame, ProcessLOD)
	end
end

function NS:AddToLOD(addon, func)
	if LODFrames[addon] then
		NS:DebugPrint("NS:AddToLOD:", "addon exists!", addon)
		return
	end
	LODFrames[addon] = { OnLoad=func }
	LODCount = LODCount + 1
	NS:RegisterEvent("ADDON_LOADED", LodFrame, ProcessLOD)
end
