--------------------------------------------------------------------------------
-- Copyright  2021								|
-- All Rights Reserved.								|
-- 										|
-- Permission to use, copy, modify, and distribute this software in any way	|
-- is not granted without expressed written agreement. This copyright applies	|
-- to the all code, data and artwork related to this project and all associated	|
-- projects published by the author.						|
--										|
-- More information on copyright is available from sources on-line.		|
-- Use an actual factual one!							|
--------------------------------------------------------------------------------

local addonName, NS = ...

local Lib = NS.FizzleLib

--------------------------- Slider -------------------------------------------------
local function Slider_Update_Color(self, value)
	if self.scale then
		value = value * self.scale
	end
--	self.Thumb:SetVertexColor(NS.Lib:GetColorMix(value, self.min, self.max, self.lowmid, self.highmid, self.lowcolor, self.midcolor, self.highcolor, self.defaultcolor))
	self.Thumb:SetVertexColor(NS.Lib:GetColorMix(value, self.min, self.max, self.lowcolor, self.midcolor, self.highcolor, self.lowmid, self.highmid))
end

local function GetSetting(self, init)
	local setting = self.Data[self.Index] or 0
	if self.scale then
		setting = setting * self.scale
	end
	local min, max = self:GetMinMaxValues()
	if setting < min or setting > max then
		if init then
			setting = min
		else
			return
		end
	end
	return setting
end

local function Slider_Init(self)--, data, index)
	local setting = GetSetting(self, true)
	if not setting then
		return
	end
	self:SetValue(setting)
	if self.ThumbOverlay then
		Slider_Update_Color(self, setting)
	end
end


local function Slider_Update(self)
	if not GetSetting(self) then
		return
	end
	local value
	if self.scale then
		value = self:GetValue()
	else
		value = floor(self:GetValue())
	end
	value = floor(value)
	self.editbox:SetText(value)
	if self.scale then
		value = value / self.scale
	end
	self.Data[self.Index] = value
	if self.ThumbOverlay then
		Slider_Update_Color(self, value)
	end
	if self.PostUpdate then
		self:PostUpdate()--value)
	end
end

local function Slider_UpdateFromEditBox(self)
	local parent = self:GetParent()
	local value = self:GetNumber()
	if not value then value = 0 end
	local min, max = parent:GetMinMaxValues()
	if parent.minlock and value < min then 
		value = min 
	end
	if parent.maxlock and value > max then 
		value = max 
	end
	value = floor(value)
	self:SetText(value)
	if value >= min and value <= max then
		parent:SetValue(value)
	end
	self:ClearFocus()
	if parent.scale then
		value = value / parent.scale
	end
	parent.Data[parent.Index] = value
	if parent.PostUpdate then
		parent:PostUpdate()--value)
	end
end

function Lib:CreateSlider(name, parent, data, index, min, minlock, max, maxlock, step, width, height, editboxw, editboxh, scale)
	local backdrop = {
		bgFile = "Interface\\Buttons\\UI-SliderBar-Background",
		edgeFile = "Interface\\Buttons\\UI-SliderBar-Border",
		tile = true,
		tileSize = 8,
		edgeSize = 8,
		insets = {
			left = 3,
			right = 3,
			top = 6,
			bottom = 6,
		},
	}
	local editbackdrop = {
		bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
		edgeFile = "Interface\\ChatFrame\\ChatFrameBackground",
		tile = true, 
		edgeSize = 1, 
		tileSize = 5,
	}
	editboxw = editboxw or 30
	editboxh = editboxh or 12
	local f = CreateFrame("Slider", "$parent"..name, parent, BackdropTemplateMixin and "BackdropTemplate")
	f:SetOrientation("HORIZONTAL")
	f:SetSize(width, height)
	f:SetHitRectInsets(0, 0, -10, -10)
	f:EnableMouse(true)
	f:SetBackdrop(backdrop)
	f.scale = scale
--	f.low = Lib:CreateFont("Min", f, min, "LEFT", "CENTER")
	f.low = Lib:CreateFont("Min", f, min, "LEFT", "MIDDLE")
	f.low:SetPoint("TOPLEFT", f, "BOTTOMLEFT", 2, 2)
--	f.high = Lib:CreateFont("Max", f, max, "RIGHT", "CENTER")
	f.high = Lib:CreateFont("Max", f, max, "RIGHT", "MIDDLE")
	f.high:SetPoint("TOPRIGHT", f, "BOTTOMRIGHT", -2, 2)
	f:SetThumbTexture("Interface\\Buttons\\UI-SliderBar-Button-Horizontal")
	f.min = min
	f.max = max
	f:SetMinMaxValues(f.min, f.max)
	f.minlock = minlock
	f.maxlock = maxlock
	f:SetValueStep(step)
	f:SetScript("OnValueChanged", function(self)
		Slider_Update(self)
	end)
	f.editbox = CreateFrame("EditBox", "$parentEditBox", f, BackdropTemplateMixin and "BackdropTemplate")
	f.editbox:SetFont(Lib.Font, Lib.FontSize, "")
	f.editbox:SetAlpha(0.99)
	f.editbox:SetBackdrop(editbackdrop)
	f.editbox:SetBackdropColor(0, 0, 0, 0)
	f.editbox:SetBackdropBorderColor(0.3, 0.3, 0.30, 0.80)
	f.editbox:SetMaxLetters(8)
	f.editbox:SetAutoFocus(false)
	f.editbox:SetJustifyH("CENTER")
	f.editbox:ClearAllPoints()
	f.editbox:SetPoint("TOP", f, "BOTTOM", 0, 1)
	f.editbox:SetTextColor(1, 1, 0)
	f.editbox:SetScript("OnEscapePressed", function(self)
		self:SetText(self:GetParent():GetValue())
		self:ClearFocus()
	end)
	f.editbox:SetScript("OnEnterPressed", function(self)
		Slider_UpdateFromEditBox(self)
	end)
	f:HookScript("OnEnable", function(self)
		self:SetAlpha(1)
		self.editbox:SetEnabled(true)
		if self.SliderThumb then
			self:SetThumbTexture(self.SliderThumb)
		end
	end)
	f:HookScript("OnDisable", function(self)
		self:SetAlpha(0.5)
		self.editbox:SetEnabled(false)
		if self.SliderThumbDisabled then
			self:SetThumbTexture(self.SliderThumbDisabled)
		end
	end)

	function f:SetEditBoxPos(point, xos, yos)
		Lib:SetRelativePoint(self.editbox, point, xos, yos)
	end
	function f:GetEditboxWidth()
		return f.editbox:GetWidth()
	end
	function f:GetEditboxHeight()
		return f.editbox:GetHeight()
	end
	function f:SetEditboxSize(width, height)
		self.editbox:SetSize(width, height)
	end
	Lib:AddLabelFuncs(f)
	f:SetEditboxSize(editboxw, editboxh)
	Lib.InitData(f, data, index, Slider_Init)
	function f:Init()
		Lib.InitData(self, self.Data, self.Index, Slider_Init)
	end
	return f
end

function Lib:CreateDiamondSlider(name, parent, data, index, min, minlock, max, maxlock, step, width, height, editboxw, editboxh, scale)
	local f = Lib:CreateSlider(name, parent, data, index, min, minlock, max, maxlock, step, width, height, editboxw, editboxh, scale)
	f.SliderThumb = Lib.TexturePath.."SliderButton"
	f.SliderThumbDisabled = Lib.TexturePath.."SliderButtonDisabled"
	f:SetThumbTexture(f.SliderThumb)
	f.Thumb = f:GetThumbTexture()
	f.Thumb:SetSize(12, 12)
	f.ThumbOverlay = f:CreateTexture()
	f.ThumbOverlay:SetAllPoints(f.Thumb)
	f.ThumbOverlay:SetTexture(Lib.TexturePath.."SliderButtonOverly")
	function f:SetColors(lowmid, highmid, lowcolor, midcolor, highcolor, defaultcolor)
		self.lowmid = lowmid
		self.highmid = highmid
		self.lowcolor = lowcolor
		self.midcolor = midcolor
		self.highcolor = highcolor
		self.defaultcolor = defaultcolor
		Slider_Update_Color(self, self.Data[self.Index])
	end
		
	Slider_Update_Color(f, data[index])
	return f
end

